@@ -19,18 +19,27 @@
 #ifndef STRUTL_H
 #define STRUTL_H
 
+#include <apt-pkg/string_view.h>
+#include <cstddef>
 #include <cstring>
+#include <ctime>
 #include <iostream>
 #include <limits>
 #include <string>
 #include <vector>
-#include <apt-pkg/string_view.h>
-#include <stddef.h>
-#include <time.h>
 
 #include "macros.h"
 
 
+namespace {
+   struct FreeDeleter {
+      void operator()(void *p) {
+         free(p);
+      }
+   };
+}
+
+
 namespace APT {
    namespace String {
       APT_PUBLIC std::string Strip(const std::string &s);
